function [u , X , Psi , flag , iterations] = scale_linear_program(Phi , a , F , method , disp)
%% Header
%
% Filename:  scale_linear_program.m
% Created:   01/08/2014
% Modified:  01/08/2014 "Created"
%            01/22/2015 "Major Update"
%
% Author(s): Chae Clark
%            cclark18@math.umd.edu
%            Applied Mathematics & Statistics, and Scientific Computing
%            Department of Mathematics
%            University of Maryland, College Park
%           
% Description:
% This function atempts to compute the scaling wieghts of the given frame
% through linear programming. For more information on this linear
% program, reference "PAPER," by Chae A. Clark and Kasso A. Okoudjou.
%
% Toolboxes:
% _auxiliary_files (provided with package)
% Optimization
%
% Input:
% Phi        - the frame to be scaled
% 
% a          - the cost coefficients used in the objective function
%
% F          - the transformed frame
%            - generating the transformation F(Phi) can be costly, so
%             compute before calling this function if possible
%
% method     - the optimization method used to solve the linear program
%             (a) 'simplex' (uses the Simplex method)
%             (b) 'interior-point' [Default] (uses an interior-point method)
%
% disp       - what is displayed during the execution of this function
%             (a) 'off'/'none' [Default] (no display)
%             (b) 'iter' (output of each iteration)
%             (c) 'final' (displays the final result)
%
% Output:
% u          - the solution to the linear program
%            - if the problem in infeasible, an empty vector is returned
%
% X          - the scaling matrix
%            - if the problem is infeasible, and empty vector is returned
%
% Psi        - the scaled frame Phi*X
%            - if the problem is infeasible, and empty vector is returned
%
% flag       - the output flag from the "linprog" function
%
% iterations - the number of iterations required to converge to a solution
%
% Copyright 2015 by Chae A. Clark. All Rights Reserved.
%

%% Preprocessing

if nargin < 1
    error('You must input a frame.')
end

% Collect Frame Size
[~,M] = size(Phi);

if nargin < 2
    a = ones(M,1);
end

if nargin < 3
    F = frame_transform(Phi);
end
[d] = size(F,1);

if nargin < 4
    method = 'interior-point';
end

if nargin < 5
    disp = 'off';
end

%% Optimization

% Options
options = optimoptions(@linprog,'Display',disp,'Algorithm',method);

% Linear System
L = [F;ones(1,M)];
b = [zeros(d,1);1];
ub = ones(M,1);
lb = 0*ones(M,1);

% Program
[u,~,flag,output] = linprog(a,[],[],L,b,lb,ub,[],options);

%% Postprocessing

if flag ~= 1
    u = [];
    X = [];
    Psi = [];
    iterations = output.iterations;
    return
else
    X = diag(sqrt(u));
    Psi = Phi*X;
    iterations = output.iterations;
end

end